@@ -10,14 +10,16 @@ function(object)
 se <- TRUE
 splineInt <- TRUE
 options(warn=0)
+
 X <- object$X
+#collapse X
+#X.full <- object$X
 
 max.it <- apply(X,2,max,na.rm=TRUE)                               #maximum item raw score without NA
 rp <- rowSums(X,na.rm=TRUE)                                       #person raw scores
 maxrp <- apply(X,1,function(x.i) {sum(max.it[!is.na(x.i)])})      #maximum item raw score for person i 
 TFrow <- ((rp==maxrp) | (rp==0))
 pers.exe <- (1:dim(X)[1])[TFrow]                                  #persons excluded from estimation due to 0/full
-#pers.notexe <- (1:dim(X)[1])[!TFrow]
 X.dummy <- X
 if (length(pers.exe) > 0) X <- X[-pers.exe,] 
 
@@ -26,10 +28,21 @@ if (any(is.na(X))) {
   dichX <- ifelse(is.na(X),1,0)
   strdata <- apply(dichX,1,function(x) {paste(x,collapse="")})
   gmemb <- as.vector(data.matrix(data.frame(strdata)))
+  gmemb1 <- gmemb
 } else {
   gmemb <- rep(1,dim(X)[1])
+  gmemb1 <- gmemb
 }
 
+mt_vek <- apply(X,2,max,na.rm=TRUE)             #number of categories - 1 for each item
+mt_ind <- rep(1:length(mt_vek),mt_vek)          #index i for items
+
+indvec <- unlist(tapply(1:dim(X)[1], gmemb, function(p.ind) {        #index vector for X collapsing
+                      !duplicated(rowSums(X[p.ind,],na.rm = TRUE))}))
+n.pall <- as.vector(unlist(tapply(1:dim(X)[1], gmemb, function(nr) {
+                      table(rowSums(X[nr,], na.rm = TRUE))})))
+X <- X[indvec,]                                      #collapsing X
+
 beta.all <- object$betapar
 
 if (!is.null(object$ngroups))
@@ -38,12 +51,17 @@ if (!is.null(object$ngroups))
 if (is.null(object$mpoints))  { mpoints <- 1
 } else {mpoints <- object$mpoints}
   
-mt_vek <- apply(X,2,max,na.rm=TRUE)             #number of categories - 1 for each item
-mt_ind <- rep(1:length(mt_vek),mt_vek)          #index i for items
 r.pall <- rowSums(X,na.rm=TRUE)                 #person raw scores
 
+
 X01 <- object$X01
 if (length(pers.exe) > 0) X01 <- X01[-pers.exe,]   #if persons excluded due to 0/full response
+
+##########################
+X01 <- X01[indvec,]                             #collapsed version
+gmemb <- gmemb[indvec]                         #collapsed version
+##############################
+
 rowvec <- 1:(dim(X01)[1])
 
 fitlist <- tapply(rowvec,gmemb,function(rind) {         #list with nlm outputs
@@ -63,7 +81,7 @@ fitlist <- tapply(rowvec,gmemb,function(rind) {         #list with nlm outputs
     theta <- rep(0,length(r.p))
     
     #==================== ML routines ===================================
-    jml.rasch <- function(theta)         #efficient ML for RM only
+    jml.rasch <- function(theta)         #fast ML for RM only
     { 
       ksi <- exp(theta)
       denom <- 1/exp(-beta)              #-beta instead of beta since beta are easiness parameter
@@ -74,21 +92,19 @@ fitlist <- tapply(rowvec,gmemb,function(rind) {         #list with nlm outputs
     jml <- function(theta)               #ML for all other models
     {
         t1t2.list <- tapply(1:(dim(X01beta)[2]),mt_ind, function(xin) {
-                     #print(xin)
-                     xb <- (t(X01beta)[xin,]) #0/1 responses and beta parameters
+                     xb <- (t(X01beta)[xin,])          #0/1 responses and beta parameters
                      if (is.vector(xb)) xb <- t(as.matrix(xb))   #if Rasch 
-                     #FIXME: nicht sicher, aber !is.na(beta)
                      beta.i <- c(0,xb[,dim(xb)[2]])    #item parameter with 0 
-                     x01.i <- (xb[,1:(dim(xb)[2]-1)])    #0/1 matrix for item i without beta
+                     x01.i <- (xb[,1:(dim(xb)[2]-1)])  #0/1 matrix for item i without beta
                      if (is.vector(x01.i)) x01.i <- t(as.matrix(x01.i))
                      cat0 <- rep(0,dim(x01.i)[2])
                      cat0[colSums(x01.i)==0] <- 1
-                     x01.i0 <- rbind(cat0,x01.i)        #appending response vector for 0th category
+                     x01.i0 <- rbind(cat0,x01.i)       #appending response vector for 0th category
                      
                      ind.h <- 0:(length(beta.i)-1)
                      theta.h <- ind.h %*% t(theta)
                      term1 <- (theta.h+beta.i)*x01.i0 
-                     t1.i <- sum(colSums(term1))        #sum over categories and persons
+                     t1.i <- sum(colSums(term1))       #sum over categories and persons
                                                       
                      term2 <- exp(theta.h+beta.i)
                      t2.i <- sum(log(colSums(term2)))   #sum over categories and persons
@@ -103,9 +119,9 @@ fitlist <- tapply(rowvec,gmemb,function(rind) {         #list with nlm outputs
         
     #==================== call optimizer =================================
     if (object$model == "RM") {
-      fit <- nlm(jml.rasch,theta,hessian=se)
+      fit <- nlm(jml.rasch,theta,hessian=se,iterlim=1000)
     } else {
-      fit <- nlm(jml,theta,hessian=se)
+      fit <- nlm(jml,theta,hessian=se,iterlim=1000)
     }
     #fit2 <- optim(theta,jml.rasch,method="BFGS",hessian=TRUE)
     
@@ -156,16 +172,35 @@ if (splineInt) {                                           #cubic spline interpo
    
 } 
 
+
 #---------labels----------------------
 names(thetapar) <- names(se.theta) <- paste("NAgroup",1:length(thetapar),sep="")
-thetaind <- rownames(X)
-for (i in 1:length(thetapar)) {names(thetapar[[i]]) <- names(se.theta[[i]]) <- thetaind[gmemb==i] }
+#thetaind <- rownames(X)
+#for (i in 1:length(thetapar)) {names(thetapar[[i]]) <- names(se.theta[[i]]) <- thetaind[gmemb==i] }
 #--------end labels------------------
+
+#---------expand theta and se.theta, labeling -------------------
+for (i in unique(gmemb)) {
+  o.r <- rowSums(object$X[gmemb1==i,], na.rm = TRUE)             #orginal raw scores
+  c.r <- rowSums(X[gmemb==i,], na.rm = TRUE)                     #collapsed raw scores
+  match.ind <- match(o.r, c.r)
+  if (length(pers.exe) != 0) { 
+    thetapar[[i]] <- thetapar[[i]][match.ind][-pers.exe]           #de-collapse theta's
+    se.theta[[i]] <- se.theta[[i]][match.ind][-pers.exe]            #de-collapse se's
+    names(thetapar[[i]]) <- names(se.theta[[i]]) <- rownames(object$X[-pers.exe])[gmemb1==i]
+  } else {
+    thetapar[[i]] <- thetapar[[i]][match.ind]           #de-collapse theta's
+    se.theta[[i]] <- se.theta[[i]][match.ind]           #de-collapse se's
+    names(thetapar[[i]]) <- names(se.theta[[i]]) <- rownames(object$X)[gmemb1==i]
+  }
+}
+#--------------- end expand, labeling ---------------------------
+
                               
 result <- list(X=X.n,X01=object$X01,W=object$W,model=object$model,loglik=loglik,npar=npar,iter=niter,
                betapar=object$betapar,thetapar=thetapar,se.theta=se.theta,
                pred.list=pred.list,hessian=hessian,mpoints=mpoints,
-               pers.ex=pers.exe,gmemb=gmemb)
+               pers.ex=pers.exe,gmemb=gmemb1)
 class(result) <- "ppar"
 result
 }
